<?php

namespace Tablesome\Includes\Modules\Automation\Integrations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\Automation\Integrations\Cf7')) {
    class Cf7
    {

        /**
         * Define the un-supported fields in CF7
         *
         */
        public $unsupported_formats = array(
            'submit',
            'file',
        );

        public static $instance = null;

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_hooks();
            }
            return self::$instance;
        }

        public function load_hooks()
        {

            add_filter('tablesome_automation/the_triggers', array($this, 'get_triggers'), 10, 2);
            add_filter('tablesome_automation/get_form_collection', array($this, 'get_form_collection'), 10, 2);

            add_action("wpcf7_before_send_mail", array($this, 'action_wpcf7_before_send_mail'), 10, 1);

        }

        public function get_triggers($triggers)
        {
            $is_active = class_exists('WPCF7') ? true : false;

            $triggers[] = array(
                'integration' => 'cf7',
                'integration_label' => __('Contact Form 7', 'tablesome'),
                'trigger' => 'tablesome_cf7_form_submit',
                'trigger_id' => 1,
                'trigger_label' => __('On Form Submit', 'tablesome'),
                'is_active' => $is_active,
                'is_premium' => "no",
            );

            return $triggers;
        }

        public function get_forms()
        {
            $forms = array();

            $posts = get_posts(array(
                'post_type' => 'wpcf7_contact_form',
                'numberposts' => -1,
            ));

            if (empty($posts)) {
                return $forms;
            }

            foreach ($posts as $post) {
                $forms[] = array(
                    'form_id' => $post->ID,
                    'form_title' => $post->post_title . " (ID: " . $post->ID . ")",
                    'form_type' => 'cf7',
                );
            }

            return $forms;
        }

        public function get_form_collection($collection)
        {
            $forms = $this->get_forms();
            if (empty($forms)) {
                return $collection;
            }

            foreach ($forms as $form) {
                $form_id = $form['form_id'];
                // Get form fields
                $form['fields'] = $this->get_form_fields($form_id);

                $collection[] = $form;
            }

            return $collection;
        }

        public function get_form_fields($form_id, array $args = array())
        {
            if (!is_plugin_active('contact-form-7/wp-contact-form-7.php')) {
                return [];
            }

            $form = \WPCF7_ContactForm::get_instance($form_id);
            $fields_object = $form->scan_form_tags();
            $fields = $this->get_fields($fields_object);
            return $fields;
        }

        public function get_fields($fields_object)
        {
            $fields = array();
            if (empty($fields_object)) {
                return $fields;
            }
            foreach ($fields_object as $field_object) {
                $basetype = isset($field_object['basetype']) ? $field_object['basetype'] : '';
                $name = isset($field_object['name']) ? $field_object['name'] : '';
                if (!empty($name) && !in_array($basetype, $this->unsupported_formats)) {

                    $field = [
                        "id" => $name,
                        "name" => $name,
                        "type" => $basetype,
                    ];

                    $have_options = (isset($field_object['values']) && !empty($field_object['values']));
                    if (in_array($basetype, ['select', 'checkbox', 'radio']) && $have_options) {
                        $field['options'] = $this->get_formatted_options($field_object);
                    }

                    $fields[] = $field;

                }
            }
            return $fields;
        }

        public function action_wpcf7_before_send_mail($wpcf7)
        {
            error_log('*** CF7 Form Submitted ***');

            $submission = \WPCF7_Submission::get_instance();
            if (!$submission) {
                return $wpcf7;
            }

            $form_tags = $wpcf7->scan_form_tags();
            // Get all the fields types
            $fields_types = array_column($form_tags, 'basetype', 'name');

            $posted_data = $submission->get_posted_data();
            $submission_data = $this->get_formatted_posted_data($posted_data, $fields_types);

            $action_data = array(
                'integration' => 'cf7',
                'form_title' => $wpcf7->title(),
                'form_submission_id' => $wpcf7->id(),
                'form_submission_data' => $submission_data,
            );

            /**
             * Trigger form runner action when submitting a from
             */
            do_action('tablesome_automation/trigger', $action_data);

        }

        public function get_formatted_posted_data($posted_data, $fields_types = array())
        {
            $data = array();
            foreach ($posted_data as $key => $value) {
                $field_type = isset($fields_types[$key]) ? $fields_types[$key] : '';

                if (is_array($value) && !empty($value)) {
                    $value = implode(',', $value);
                } else if (is_valid_tablesome_date($value, 'Y-m-d')) {
                    $unix_timestamp = convert_tablesome_date_to_unix_timestamp($value, 'Y-m-d');
                    $unix_timestamp = $unix_timestamp * 1000; // convert to milliseconds
                }

                $data[$key] = array(
                    'value' => $value,
                    'type' => $field_type,
                    'unix_timestamp' => isset($unix_timestamp) ? $unix_timestamp : '', // use this prop when the column format type is date
                );
            }
            return $data;
        }

        public function get_formatted_options($field_object)
        {
            $options = array();
            foreach ($field_object['values'] as $value) {
                $options[] = array(
                    'value' => $value,
                    'label' => $value,
                );
            }
            return $options;
        }
    }
}
