<?php

namespace Tablesome\Includes\Modules\Automation\Integrations\Notion;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\Automation\Integrations\Notion\Database')) {
    class Database
    {
        use Properties;

        public $unsupported_types = array(
            'formula', 'relation', 'rollup', 'files', 'created_time', 'created_by', 'last_edited_time', 'last_edited_by', 'people',
        );

        public $api_key = '';
        public $api_status = false;
        public $version = '2022-02-22';

        public static $instance = null;

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_hooks();
                $api_key = get_option(TABLESOME_NOTION_API_KEY);
                $api_status = get_option(TABLESOME_NOTION_API_STATUS);

                self::$instance->api_key = $api_key ? $api_key : self::$instance->api_key;
                self::$instance->api_status = $api_status ? $api_status : self::$instance->api_status;

            }
            return self::$instance;
        }

        public function load_hooks()
        {

            add_filter('tablesome_automation/the_actions', array($this, 'get_actions'), 10, 2);

            add_filter('tablesome_automation/get_database_collection', array($this, 'get_database_collection'), 10, 2);

            add_action('tablesome_automation/add_page_to_notion_db', [self::$instance, 'add_page'], 10, 2);
        }

        public function add_api($api_key)
        {
            $this->api_key = $api_key;
            update_option(TABLESOME_NOTION_API_KEY, $api_key);
        }

        public function remove_api_data()
        {
            delete_option(TABLESOME_NOTION_API_KEY);
            delete_option(TABLESOME_NOTION_API_STATUS);
            delete_option(TABLESOME_NOTION_API_STATUS_MESSAGE);
        }

        public function ping()
        {
            $args = array(
                'headers' => $this->get_api_headers(),
                'body' => json_encode(array(
                    'filter' => array(
                        'value' => 'database',
                        'property' => 'object',
                    ),
                )),
            );
            $url = 'https://api.notion.com/v1/search';
            $response = wp_remote_post($url, $args);
            $data = json_decode(wp_remote_retrieve_body($response), true);

            if ($response['response']['code'] != 200) {
                $message = isset($data['message']) ? $data['message'] : 'The API key is invalid.';

                update_option(TABLESOME_NOTION_API_STATUS, false);
                update_option(TABLESOME_NOTION_API_STATUS_MESSAGE, $message);

                return array(
                    'status' => false,
                    'message' => $message,
                );
            }
            update_option(TABLESOME_NOTION_API_STATUS, true);
            update_option(TABLESOME_NOTION_API_STATUS_MESSAGE, 'Connected');

            return array(
                'status' => true,
                'message' => 'Connected',
            );
        }

        public function get_actions($actions)
        {
            $actions[] = array(
                'integration' => 'notion',
                'integration_label' => __('Notion', 'tablesome'),
                'is_active' => $this->api_status,
                'is_premium' => false,
                'actions' => array(
                    array(
                        'id' => 3,
                        'name' => 'add_page',
                        'label' => __('Add Record to Notion DB', 'tablesome'),
                    ),
                ),
            );
            return $actions;
        }

        public function get_api_headers()
        {
            return array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Notion-Version' => $this->version,
            );
        }

        public function get_database_collection()
        {
            $status = get_option(TABLESOME_NOTION_API_STATUS);
            $message = get_option(TABLESOME_NOTION_API_STATUS_MESSAGE);

            $api_not_configured = empty($status) && empty($message);

            if ($api_not_configured) {
                $message = 'Please configure Notion API in Tablesome for this action to work.';
            }

            return array(
                'databases' => $this->get_all_databases(),
                'api' => array(
                    'status' => !$status ? false : true,
                    'message' => $message,
                    'redirect_url' => admin_url('edit.php?post_type=' . TABLESOME_CPT . '&page=tablesome-settings#tab=integrations/notion'),
                    'share_database_url' => 'https://developers.notion.com/docs/getting-started#step-2-share-a-database-with-your-integration',
                ),
            );
        }

        public function get_all_databases()
        {
            if (!$this->api_status || empty($this->api_key)) {
                return array();
            }

            $args = array(
                'headers' => $this->get_api_headers(),
                'body' => json_encode(array(
                    'filter' => array(
                        'value' => 'database',
                        'property' => 'object',
                    ),
                )),
            );
            $url = 'https://api.notion.com/v1/search';
            $response = wp_remote_post($url, $args);

            if (is_wp_error($response)) {
                return array();
            }

            if ($response['response']['code'] != 200) {
                return [];
            }
            $data = json_decode(wp_remote_retrieve_body($response), true);
            $results = $data['results'];

            $databases = array();

            foreach ($results as $result) {
                $properties = isset($result['properties']) ? $result['properties'] : array();

                $fields = [];

                if (!empty($properties)) {
                    foreach ($properties as $property) {

                        if (in_array($property['type'], $this->unsupported_types)) {
                            continue;
                        }

                        $fields[] = array(
                            'id' => $property['id'],
                            'name' => $property['name'],
                            'type' => $property['type'],
                        );
                    }
                }
                $databases[] = array(
                    'id' => $result['id'],
                    'name' => isset($result['title'][0]) ? $result['title'][0]['plain_text'] : 'Untitled',
                    'fields' => $fields,
                    'url' => $result['url'],
                    'archived' => $result['archived'],
                );
            }
            return $databases;
        }

        public function get_database_by_id($database_id)
        {
            if (empty($database_id) || !$this->api_status || empty($this->api_key)) {
                return false;
            }

            $payload = array(
                'headers' => $this->get_api_headers(),
            );
            $url = "https://api.notion.com/v1/databases/{$database_id}";
            $response = wp_remote_get($url, $payload);
            if ($response['response']['code'] != 200) {
                return [];
            }
            $data = json_decode(wp_remote_retrieve_body($response), true);

            return $data;
        }

        public function add_page($args)
        {
            if (!$this->api_status || empty($this->api_key)) {
                return false;
            }

            $action_meta = $args['action_meta'];
            $form_data = isset($args['form_submitted_data']['form_submission_data']) ? $args['form_submitted_data']['form_submission_data'] : [];

            $database_id = isset($action_meta['database_id']) ? $action_meta['database_id'] : '';
            $match_fields = isset($action_meta['match_fields']) ? $action_meta['match_fields'] : [];

            if (empty($database_id) || empty($match_fields)) {
                return;
            }

            $database = $this->get_database_by_id($database_id);

            if (empty($database)) {
                return;
            }

            $data = $this->get_matched_property_values($match_fields, $form_data, $database);

            if (empty($data)) {
                return;
            }

            $payload = [
                'headers' => $this->get_api_headers(),
                'body' => json_encode(
                    [
                        'parent' => [
                            'type' => 'database_id',
                            'database_id' => $database_id,
                        ],
                        'properties' => $data,
                    ]
                ),
            ];

            $url = 'https://api.notion.com/v1/pages';
            $response = wp_remote_post($url, $payload);

            if ($response['response']['code'] != 200) {
                return [];
            }

            $response_data = json_decode(wp_remote_retrieve_body($response), true);
            return $response_data;
        }

        public function get_matched_property_values($match_fields, $form_data, $database)
        {

            $data = array();
            foreach ($match_fields as $match_field) {
                $property_id = isset($match_field['property_id']) ? $match_field['property_id'] : '';
                $field_name = isset($match_field['field_name']) ? $match_field['field_name'] : '';

                if (empty($property_id)) {
                    continue;
                }

                $property = $this->get_property($property_id, $database);

                if (empty($property)) {
                    continue;
                }

                $value = isset($form_data[$field_name]['value']) ? $form_data[$field_name]['value'] : '';
                $property_values = $this->get_property_values($property, $value);

                if (empty($property_values)) {
                    continue;
                }

                $data[$property_id] = $property_values;
            }
            return $data;
        }

    }
}
