<?php

namespace Tablesome\Includes\Modules\Automation\Integrations;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\Automation\Integrations\WP_Forms')) {
    class WP_Forms
    {
        public $unsupported_formats = array(
            'mailchimp',
            'password',
            'file-upload',
            'divider',
            'html',
            'pagebreak',
            'signature',
            'captcha',
            'net_promoter_score',
            'captcha_recaptcha',
        );

        public static $instance = null;

        public static function get_instance()
        {
            if (is_null(self::$instance)) {
                self::$instance = new self();
                self::$instance->load_hooks();
            }

            return self::$instance;
        }

        public function load_hooks()
        {
            add_filter('tablesome_automation/the_triggers', array($this, 'get_triggers'), 10, 2);
            add_filter('tablesome_automation/get_form_collection', array($this, 'get_form_collection'), 10, 2);
            add_filter('tablesome_automation/get_wpform_field_option_id_by_value', array($this, 'get_wpform_field_option_id_by_value'), 10, 2);

            add_action("wpforms_process_entry_save", array($this, 'wpforms_process_entry_save'), 10, 4);

        }

        public function get_triggers($triggers)
        {
            $is_active = class_exists('WPForms') ? true : false;

            $triggers[] = array(
                'integration' => 'wpforms',
                'integration_label' => __('WPForms', 'tablesome'),
                'trigger' => 'tablesome_wpforms_form_submit',
                'trigger_id' => 2,
                'trigger_label' => __('On Form Submit', 'tablesome'),
                'is_active' => $is_active,
                'is_premium' => "no",
            );

            return $triggers;
        }

        public function get_form_collection($collection)
        {
            $forms = $this->get_forms();
            if (empty($forms)) {
                return $collection;
            }

            foreach ($forms as $form) {
                $form['fields'] = $this->get_form_fields($form);

                // Remove the form-content property.
                if (isset($form['form_content'])) {
                    unset($form['form_content']);
                }
                $collection[] = $form;
            }
            return $collection;
        }

        public function get_forms()
        {
            $forms = array();
            $exists = function_exists('wpforms') && method_exists(wpforms()->form, 'get');
            if (!$exists) {
                return $forms;
            }

            $posts = wpforms()->form->get('');

            if (empty($posts)) {
                return $posts;
            }
            foreach ($posts as $post) {
                $forms[] = array(
                    'form_id' => $post->ID,
                    'form_title' => $post->post_title . " (ID: " . $post->ID . ")",
                    'form_type' => 'wpforms',
                    'form_content' => $post->post_content,
                );
            }
            return $forms;
        }

        public function get_form_fields($form, array $args = array())
        {
            $form_data = !empty($form['form_content']) ? wpforms_decode($form['form_content']) : '';
            $form_fields = isset($form_data['fields']) ? $form_data['fields'] : [];

            if (empty($form_fields)) {
                return [];
            }
            $fields = array();
            foreach ($form_fields as $form_field) {
                $type = isset($form_field['type']) ? $form_field['type'] : '';
                $label = isset($form_field['label']) && !empty($form_field['label']) ? $form_field['label'] : 'label-' . $form_field['id'];
                if (!in_array($type, $this->unsupported_formats)) {

                    $field = [
                        "id" => $form_field['id'],
                        "name" => $label,
                        "type" => $type,
                    ];

                    $have_options = (isset($form_field['choices']) && !empty($form_field['choices']));
                    if ($have_options) {
                        $field['options'] = $this->get_formatted_options($form_field);
                    }
                    $fields[] = $field;
                }
            }
            return $fields;
        }

        public function wpforms_process_entry_save($fields, $entry, $form_id, $form_data)
        {

            // error_log('$fields : ' . print_r($fields, true));
            // error_log('$entry : ' . print_r($entry, true));
            // error_log('$form_data : ' . print_r($form_data, true));

            $submission_data = $this->get_formatted_posted_data($fields, $entry, $form_id, $form_data);

            $action_data = array(
                'integration' => 'wpforms',
                'form_submission_id' => $form_id,
                'form_title' => $form_data['settings']['form_title'],
                'form_submission_data' => $submission_data,
                'wpforms_data' => $form_data,
            );

            /**
             * Trigger action when submitting a from
             */
            do_action('tablesome_automation/trigger', $action_data);
        }

        public function get_formatted_posted_data($fields, $entry, $form_id, $form_data)
        {
            $data = array();
            if (empty($fields)) {
                return $data;
            }
            foreach ($fields as $key => $field) {
                $value = isset($field['value']) ? $field['value'] : '';
                $type = isset($field['type']) ? $field['type'] : '';

                if ($type == 'date-time') {
                    /**
                     *  Issue #1093 - For supporting the date field
                     *  In WPForms, they also give the user-submitted date-time value in unix format.
                     *
                     *  Ref: https://stackoverflow.com/questions/4676195/why-do-i-need-to-multiply-unix-timestamps-by-1000-in-javascript
                     */
                    $unix_timestamp = $field['unix'];
                    $unix_timestamp = $unix_timestamp * 1000;
                } else if ($type == 'checkbox' || $type == 'select') {
                    $value = explode("\n", $value);
                    $value = is_array($value) && !empty($value) ? implode(',', $value) : $value;
                }

                $data[$key] = array(
                    'value' => $value,
                    'type' => $type,
                    'unix_timestamp' => isset($unix_timestamp) ? $unix_timestamp : '', // use this prop when the column format type is date
                );
            }
            return $data;
        }

        public function get_formatted_options($form_field)
        {
            $options = array();
            foreach ($form_field['choices'] as $id => $choice_data) {
                $options[] = array(
                    'value' => $id,
                    'label' => $choice_data['label'],
                );
            }
            return $options;
        }

        public function get_wpform_field_option_id_by_value($props, $form_submitted_data)
        {
            $selected_option_id = $props['value'];
            $field_id = $props['field'];

            $wpforms_data = isset($form_submitted_data['wpforms_data']) ? $form_submitted_data['wpforms_data'] : [];

            $field = isset($wpforms_data['fields'][$field_id]) ? $wpforms_data['fields'][$field_id] : [];
            $choices = isset($field['choices']) ? $field['choices'] : [];
            if (empty($field) || empty($choices)) {
                return $selected_option_id;
            }

            foreach ($choices as $id => $choice) {
                if ($choice['label'] == $props['submitted_value']) {
                    $selected_option_id = $id;
                    break;
                }
            }

            return $selected_option_id;
        }
    }
}
